/**********************************************************************
Copyright (c) 2009 Clive Cox and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.cache.memcache;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;

import net.spy.memcached.AddrUtil;
import net.spy.memcached.MemcachedClient;

import org.datanucleus.OMFContext;
import org.datanucleus.PersistenceConfiguration;
import org.datanucleus.cache.CachedPC;
import org.datanucleus.cache.Level2Cache;
import org.datanucleus.exceptions.NucleusException;

public class MemcacheLevel2Cache implements Level2Cache
{
    private MemcachedClient client;

    private String keyPrefix = "datanucleus:"; // prefix to ensure keys can be shared with other memcache
                                               // objects

    private int expireSeconds = 0;

    public MemcacheLevel2Cache(OMFContext omfCtx)
    {
        PersistenceConfiguration conf = omfCtx.getPersistenceConfiguration();
        String servers = conf.getStringProperty("datanucleus.cache.level2.memcache.servers");
        String keyPrefix = conf.getStringProperty("datanucleus.cache.level2.memcache.keyprefix");
        String expireStr = conf.getStringProperty("datanucleus.cache.level2.memcache.expireSeconds");
        if (keyPrefix != null)
        {
            this.keyPrefix = keyPrefix;
        }

        try
        {
            client = new MemcachedClient(AddrUtil.getAddresses(servers));
            if (expireStr != null && !"".equals(expireStr))
                expireSeconds = Integer.parseInt(expireStr);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            throw new NucleusException("Cant create cache", e);
        }
        catch (NumberFormatException ex)
        {
            throw new NucleusException("Cant create cache: Bad expireSeconds value:" + expireStr, ex);
        }
    }

    public void clear()
    {
        client.flush();
    }

    public void close()
    {
        client.shutdown();
    }

    public boolean containsOid(Object oid)
    {
        return get(oid) != null;
    }

    public void evict(Object oid)
    {
        client.delete(keyPrefix + oid.toString());
    }

    public void evictAll()
    {
        throw new UnsupportedOperationException("evictAll() method not yet supported by Memcache plugin");
    }

    public void evictAll(Object[] oids)
    {
        if (oids == null)
        {
            return;
        }

        for (int i = 0; i < oids.length; i++)
        {
            evict(oids[i]);
        }
    }

    public void evictAll(Collection oids)
    {
        if (oids == null)
        {
            return;
        }

        Iterator iter = oids.iterator();
        while (iter.hasNext())
        {
            evict(iter.next());
        }
    }

    public void evictAll(Class arg0, boolean arg1)
    {
        throw new UnsupportedOperationException("evictAll(Class,Boolean) method not supported by Memcache plugin");
    }

    public CachedPC get(Object oid)
    {
        return (CachedPC) client.get(keyPrefix + oid.toString());
    }

    public int getNumberOfPinnedObjects()
    {
        throw new UnsupportedOperationException("getNumberOfPinnedObjects() method not supported by Memcache plugin");
    }

    public int getNumberOfUnpinnedObjects()
    {
        throw new UnsupportedOperationException("getNumberOfUnpinnedObjects() method not supported by Memcache plugin");
    }

    /**
     * Return a dummy value
     */
    public int getSize()
    {
        return -1;
    }

    public boolean isEmpty()
    {
        throw new UnsupportedOperationException("isEmpty() method not supported by Memcache plugin");
    }

    public void pin(Object arg0)
    {
        throw new UnsupportedOperationException("pin(Object) method not supported by Memcache plugin");
    }

    public void pinAll(Collection arg0)
    {
        throw new UnsupportedOperationException("pinAll(Collection) method not supported by Memcache plugin");
    }

    public void pinAll(Object[] arg0)
    {
        throw new UnsupportedOperationException("pinAll(Object[]) method not supported by Memcache plugin");
    }

    public void pinAll(Class arg0, boolean arg1)
    {
        throw new UnsupportedOperationException("pinAll(Class,boolean) method not supported by Memcache plugin");
    }

    public CachedPC put(Object oid, CachedPC pc)
    {
        if (oid == null || pc == null)
        {
            return null;
        }

        client.set(keyPrefix + oid.toString(), expireSeconds, pc);

        return pc;
    }

    public void unpin(Object arg0)
    {
        throw new UnsupportedOperationException("unpin(Object) method not supported by Memcache plugin");
    }

    public void unpinAll(Collection arg0)
    {
        throw new UnsupportedOperationException("unpinAll(Collection) method not supported by Memcache plugin");
    }

    public void unpinAll(Object[] arg0)
    {
        throw new UnsupportedOperationException("unpinAll(Object[]) method not supported by Memcache plugin");
    }

    public void unpinAll(Class arg0, boolean arg1)
    {
        throw new UnsupportedOperationException("unpinAll(Class,boolean) method not supported by Memcache plugin");
    }
}